"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.login = exports.loginClient = exports.pwHash = exports.logout = exports.reqReadEntities = exports.reqEntityConf = exports.presenceFor = exports.getChannelTypeFromChannel = exports.fromChannel = exports.req = exports.unsubEntityUpdates = exports.subEntityUpdates = exports.ridToBusChannel = exports.confUpdatesChannel = exports.unsub = exports.sub = exports.pub = void 0;

var _extendedDsClient = _interopRequireWildcard(require("extended-ds-client"));

var _sha = _interopRequireDefault(require("crypto-js/sha256"));

var R = _interopRequireWildcard(require("ramda"));

var _jstiip = _interopRequireDefault(require("jstiip"));

var _subSeconds = _interopRequireDefault(require("date-fns/fp/subSeconds"));

var _get = _interopRequireDefault(require("lodash/fp/get"));

var _dmUtils = require("./dm-utils");

var _transforms = require("./transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Publishes a message through the backend.
 * @func
 * @static
 * @param {string} channel The channel to publish on.
 * @param {Array} pl The payload to send.
 * @param {string} sig The signal data to apply to the published message.
 */
var pub = function pub(channel) {
  var pl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  var sig = arguments.length > 2 ? arguments[2] : undefined;
  var msg = new _jstiip["default"]({
    pl: pl
  });
  if (sig) msg.sig = sig;
  var msgOut = msg.toJS();

  _extendedDsClient.ds.client.event.emit(channel, msgOut); // console.log('PUB:', channel, msgOut);

};
/**
 * Low-level subscribe to a channel on the backend. Recommended to use fromChannel instead.
 * @func
 * @static
 * @param {function} callback The callback to trigger when a message arrives.
 * @param {string} channel The channel to subscribe to.
 */


exports.pub = pub;
var sub = R.curry(function (callback, channel) {
  return _extendedDsClient.ds.client.event.subscribe(channel, callback);
});
/**
 * Low-level unsubscribe from a channel on the backend (See fromChannel).
 * @func
 * @static
 * @param  {string} channel - The channel to unsubscribe from.
 * @param  {function} [callback] - The callback that was previously registered with subscribe. If
 *  the callback is omitted, all subscriptions for the channel will be removed.
 */

exports.sub = sub;

var unsub = function unsub(channel, callback) {
  return _extendedDsClient.ds.client.event.unsubscribe(channel, callback);
};
/**
 * [curried] From type & rid, get the channel id for updates on that record
 * @param {string} type Type of entity
 * @param {string} rid Entity rid
 * @returns {string} A bus channel id for entity updates
 */


exports.unsub = unsub;
var confUpdatesChannel = R.curry(function (type, rid) {
  return "conf/".concat(type, "/update/").concat(rid);
});
/**
 * From a rid, get the channel id on the bus
 * @param {string} rid Channel entity rid
 * @returns {string} A bus channel id for channel data
 */

exports.confUpdatesChannel = confUpdatesChannel;

var ridToBusChannel = function ridToBusChannel(rid) {
  return "data/".concat(rid);
};

exports.ridToBusChannel = ridToBusChannel;

var subEntityUpdates = function subEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (cb) {
    return R.forEach(R.o(sub(cb), ridToChannel));
  };
};

exports.subEntityUpdates = subEntityUpdates;

var unsubEntityUpdates = function unsubEntityUpdates(type) {
  var ridToChannel = confUpdatesChannel(type);
  return function (rids, cb) {
    return R.forEach(R.o(function (ch) {
      return unsub(ch, cb);
    }, ridToChannel))(rids);
  };
};
/**
 * Performs a request to the backend.
 * TODO: Refactor temporarily to rids:* to target all, but only for 'conf...'-requests
 * @func
 * @static
 * @param {string} id The id of the rpc
 * @param {object} args Arguments passed on to the rpc
 * @return {Promise} A Promise
 */


exports.unsubEntityUpdates = unsubEntityUpdates;
var req = R.curry(function (id, args) {
  // console.log('req:', id, args);
  return _extendedDsClient.ds.client.rpc.p.make(id, args)["catch"](function (err) {
    console.error(id, args, err);
    throw err;
  });
});
exports.req = req;

var readChannel = function readChannel(args) {
  return req('conf/readChannel', args);
};

var readChannelType = function readChannelType(args) {
  return req('conf/readChannelType', args);
};
/**
 * Main function for subscribing to and getting historic data from a channel.
 * @func
 * @static
 * @param {object} options
 * @param {string} options.rid
 * @param {boolean} options.last If to include last value
 * @param {number} options.tail How many historic values to fetch
 * @param {number} options.ttl Time to live as seconds
 * @param {function} options.cb Callback (for one/each value)
 * @param {boolean} options.noSub If not to subscribe
 * @param {string} options.path Extract a path from each message
 * @param {boolean} options.noTsToDate No Date conversion
 * @param {string[]} options.projection
 * @param {string} options.busChannel Raw channel (instead of rid)
 * @returns {function|undefined} Unsubscribe function if relevant
 */


var fromChannel = function fromChannel(_ref) {
  var rid = _ref.rid,
      last = _ref.last,
      tail = _ref.tail,
      ttl = _ref.ttl,
      cb = _ref.cb,
      noSub = _ref.noSub,
      path = _ref.path,
      noTsToDate = _ref.noTsToDate,
      projection = _ref.projection,
      busChannel = _ref.busChannel;
  if (rid === undefined && busChannel === undefined) return undefined;
  var callback = R.compose(cb, R.when(function () {
    return path !== undefined;
  }, (0, _get["default"])(path)), R.when(function () {
    return !noTsToDate;
  }, _dmUtils.evolveTsToDate), _dmUtils.ensureTsV3);

  if (last || tail) {
    req('data-history/read', _objectSpread({
      channel: rid
    }, ttl ? {
      from: (0, _dmUtils.dateToTs)((0, _subSeconds["default"])(ttl)(new Date()))
    } : {}, {
      limit: tail || Number(last),
      projection: projection
    })).then(function (reply) {
      return reply.reverse().forEach(callback);
    });
  }

  if (!noSub) {
    var channel = busChannel || ridToBusChannel(rid);
    sub(callback, channel);

    var unsubscribe = function unsubscribe() {
      return unsub(channel, callback);
    };

    return unsubscribe; // notify caller about a subscription made
  }

  return undefined;
};
/**
 * Get a channel's ChannelType record (from only the channel rid).
 * @func
 * @static
 * @param {type} rid A channel rid
 * @returns {Promise} A Promise of a type record
 */


exports.fromChannel = fromChannel;

var getChannelTypeFromChannel =
/*#__PURE__*/
function () {
  var _ref2 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee(rid) {
    var _ref3, _ref4, _ref4$, id, _ref5, _ref6, type;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            _context.next = 2;
            return readChannel({
              rids: [rid]
            });

          case 2:
            _ref3 = _context.sent;
            _ref4 = _slicedToArray(_ref3, 1);
            _ref4$ = _ref4[0];
            _ref4$ = _ref4$ === void 0 ? {} : _ref4$;
            id = _ref4$.type;

            if (!id) {
              _context.next = 14;
              break;
            }

            _context.next = 10;
            return readChannelType({
              rids: [],
              filter: ['=', 'id', id]
            });

          case 10:
            _ref5 = _context.sent;
            _ref6 = _slicedToArray(_ref5, 1);
            type = _ref6[0].type;
            return _context.abrupt("return", type);

          case 14:
            return _context.abrupt("return", undefined);

          case 15:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function getChannelTypeFromChannel(_x) {
    return _ref2.apply(this, arguments);
  };
}();

exports.getChannelTypeFromChannel = getChannelTypeFromChannel;

var presenceFor = function presenceFor(_ref7) {
  var id = _ref7.id,
      cb = _ref7.cb,
      noSub = _ref7.noSub;
  if (id === undefined) return undefined;
  req('presence/listClients', {}).then(function (reply) {
    return cb(reply.includes(id));
  });

  if (!noSub) {
    var callback = R.o(cb, R.prop('login'));
    var channel = "presence/".concat(id);
    sub(callback, channel);

    var unsubscribe = function unsubscribe() {
      return unsub(channel, callback);
    };

    return unsubscribe; // notify caller about a subscription made
  }

  return undefined;
};

exports.presenceFor = presenceFor;
var reqEntityConf = R.curry(function (id, entityRid) {
  return req('conf/readConfiguration', _objectSpread({
    entities: (0, _transforms.ensureArray)(entityRid),
    rids: []
  }, id ? {
    filter: ['=', 'id', id]
  } : {})).then(function (results) {
    return id && typeof entityRid === 'string' ? results[0] : results;
  });
});
exports.reqEntityConf = reqEntityConf;

var entitiesConfigsMapper = function entitiesConfigsMapper(configIds) {
  return R.compose(R.chain(function (entityRid) {
    return R.map(function (id) {
      return {
        id: id,
        entityRid: entityRid
      };
    })(configIds);
  }), R.pluck('rid'));
};

var inflateConfMap = R.map(function (obj) {
  return reqEntityConf(obj.id, obj.entityRid).then(R.compose( // R.tap(console.log),
  R.mergeLeft(obj), R.objOf('config')));
});

var injectConfsInto = function injectConfsInto(entities) {
  return R.compose(R.values, R.reduce(function (enties, _ref8) {
    var config = _ref8.config,
        id = _ref8.id,
        entityRid = _ref8.entityRid;
    if (config && R.has(entityRid)(enties)) return R.evolve(_defineProperty({}, entityRid, (0, _transforms.ensureAndEvolveKey)({}, 'configurations', R.mergeLeft(_defineProperty({}, id, config)))))(enties);
    return enties;
  }, (0, _dmUtils.formatRecords)(entities)));
};
/**
 * Abstracted req call with support for configurationsToInclude from future.
 * @func
 * @static
 * @param {object} args The arguments for the call. Supports configurationsToInclude.
 * @returns {object} Collection (list) of entities read, or one entity for specific requests.
 */


var reqReadEntities =
/*#__PURE__*/
function () {
  var _ref9 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee2(rpc, args) {
    var _args$configurationsT, configurationsToInclude, rest, reply, entities, confsToRead, confRequests, configsWithRefs, result;

    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            // TODO: when configuration duck is in place, only config rids should be put in each asset's
            // 'configurations', and configs should then be fetched separately into the configuration duck
            _args$configurationsT = args.configurationsToInclude, configurationsToInclude = _args$configurationsT === void 0 ? [] : _args$configurationsT, rest = _objectWithoutProperties(args, ["configurationsToInclude"]);
            _context2.next = 3;
            return req(rpc, rest);

          case 3:
            reply = _context2.sent;
            entities = (0, _transforms.ensureArray)(reply);
            confsToRead = entitiesConfigsMapper(configurationsToInclude)(entities); // console.log('confsToRead:', confsToRead);

            confRequests = inflateConfMap(confsToRead);
            _context2.next = 9;
            return Promise.all(confRequests);

          case 9:
            configsWithRefs = _context2.sent;
            // console.log('configsWithRefs (should be list):', configsWithRefs);
            result = injectConfsInto(entities)(configsWithRefs); // console.log('result:', result);

            return _context2.abrupt("return", Array.isArray(reply) ? result : R.prop(0, result));

          case 12:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function reqReadEntities(_x2, _x3) {
    return _ref9.apply(this, arguments);
  };
}();

exports.reqReadEntities = reqReadEntities;

var logout = function logout() {
  if (!_extendedDsClient.ds.client) return;

  _extendedDsClient.ds.client.off();

  _extendedDsClient.ds.client.close();
};
/**
 * Creates a connection to the backend server.
 * @func
 * @static
 * @param {string} serverAddress
 * @param {function} onUnexpectedError Callback triggered on general errors in the client.
 * @return {object} - An extended deepstream client
 */


exports.logout = logout;

var createConnection = function createConnection(serverAddress, onUnexpectedError) {
  console.log('createConnection:', serverAddress);
  var client = (0, _extendedDsClient["default"])(serverAddress);
  client.on('error', onUnexpectedError);
  return client;
};
/**
 * Hash a password (or other string)
 * @param {string} pw A password
 * @returns {string} A hash string
 */


var pwHash = function pwHash(pw) {
  return String((0, _sha["default"])(pw));
};
/**
 * INTERNAL helper function for logging in to an i4 server
 * @func
 * @static
 * @private
 * @param {object} client
 * @param {string} serverAddress
 * @param {string} id Client/user id
 * @param {string} password Probably hashed
 * @param {function} onConnectionError Callback invoked on a connection Error
 * @returns {object} An object with id, password and serverAddress
 */


exports.pwHash = pwHash;

var loginClient =
/*#__PURE__*/
function () {
  var _ref10 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee3(client, serverAddress, id, password, onConnectionError) {
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            _context3.next = 2;
            return client.p.login({
              id: id,
              password: password
            });

          case 2:
            client.on('connectionStateChanged', function (state) {
              // We need to set this only when we know that we got a working logged in connection
              console.log('CONNECTION STATE:', state);
              if (state === 'ERROR' && client.connectionState !== 'ERROR') onConnectionError();
              /* eslint-disable no-param-reassign */

              client.connectionState = state;
              /* eslint-enable no-param-reassign */
            });
            return _context3.abrupt("return", {
              id: id,
              password: password,
              serverAddress: serverAddress
            });

          case 4:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function loginClient(_x4, _x5, _x6, _x7, _x8) {
    return _ref10.apply(this, arguments);
  };
}();
/**
 * Function for logging in to an i4 server. Will use a singleton "global" client.
 * @param {string} serverAddress Endpoint to connect to
 * @param {string} id Client/user id
 * @param {string} password The password (probably hashed)
 * @param {function} onConnectionError Callback invoked on a connection Error
 * @param {function} onUnexpectedError Callback triggered on general errors in the client
 * @returns {type} Description
 */


exports.loginClient = loginClient;

var login =
/*#__PURE__*/
function () {
  var _ref11 = _asyncToGenerator(
  /*#__PURE__*/
  regeneratorRuntime.mark(function _callee4(serverAddress, id, password, onConnectionError, onUnexpectedError) {
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            if (_extendedDsClient.ds.client) logout();
            _extendedDsClient.ds.client = createConnection(serverAddress, onUnexpectedError);
            return _context4.abrupt("return", loginClient(_extendedDsClient.ds.client, serverAddress, id, password, onConnectionError));

          case 3:
          case "end":
            return _context4.stop();
        }
      }
    }, _callee4);
  }));

  return function login(_x9, _x10, _x11, _x12, _x13) {
    return _ref11.apply(this, arguments);
  };
}();

exports.login = login;