"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.evolveTsToDate = exports.tsToPl = exports.tsToDate = exports.dateToUtcTs = exports.dateToTs = exports.formatRecords = exports.ensureRidsInArgs = exports.ensureTsV3 = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _dataUtils = require("./data-utils");

var _transforms = require("./transforms");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * ensureTsV3 - If the old key 'ct' exists, instead add 'ts' on correct form
 * @func
 * @static
 * @param {object} msg A js object (a tiip-like message)
 * @returns {object} A new js object with key 'ts'
 */
var ensureTsV3 = function ensureTsV3(msg) {
  var ct = msg.ct;

  if (ct !== undefined) {
    if (Object.prototype.toString.call(ct) === '[object Date]') return _objectSpread({}, msg, {
      ts: ct
    });
    return _objectSpread({}, msg, {
      ts: new Date(Number(ct) * 1000).toISOString()
    });
  } // if (Object.prototype.toString.call(ts) === '[object Date]') return msg;


  return msg;
};
/**
 * Makes sure that any value is an object with key 'rids'
 * @func
 * @static
 * @param {any} a Any argument
 * @returns {object} An object with key 'rids' as empty array, if not already present
 */


exports.ensureTsV3 = ensureTsV3;
var ensureRidsInArgs = R.o((0, _transforms.ensureKey)([], 'rids'), R.when(R.isNil, R.always({})));
/**
 * Turns an array with records into a map, indexed by 'rid'
 * @func
 * @static
 * @param {array} a Array of records
 * @returns {object} Object mapping rid -> record
 */

exports.ensureRidsInArgs = ensureRidsInArgs;
var formatRecords = R.indexBy(R.prop('rid'));
/**
 * Turn a Date into a timestamp string
 * @func
 * @static
 * @param {Date} a A Date
 * @returns {string} Timestamp string
 */

exports.formatRecords = formatRecords;

var dateToTs = function dateToTs(da) {
  return da.toISOString();
};
/**
 * Translate a Date to UTC and then into a timestamp string
 * @func
 * @static
 * @param {Date} a A Date
 * @returns {string} Timestamp string (as UTC)
 */


exports.dateToTs = dateToTs;
var dateToUtcTs = R.o(dateToTs, _dataUtils.dateToUTC);
/**
 * Turn a timestamp string into a local Date object. (Make sure it has TZ set, like 'Z', last)
 * @func
 * @static
 * @param {string} ts Timestamp string
 * @returns {Date} A Date (local time)
 */

exports.dateToUtcTs = dateToUtcTs;

var tsToDate = function tsToDate(ts) {
  return new Date(ts);
};
/**
 * From a msg object, get its 'pl' array with 'ts' prepended
 * @func
 * @static
 * @param {object} msg A message object
 * @returns {array} The array 'pl' in message, prepended with the value in 'ts'
 */


exports.tsToDate = tsToDate;

var tsToPl = function tsToPl(msg) {
  return R.prepend(msg.ts, msg.pl);
};
/**
 * Transform 'ts' in an object/message with tsToDate
 * @func
 * @static
 * @param {object} a A message object
 * @returns {object} A new object with 'ts' transformed to a local Date
 */


exports.tsToPl = tsToPl;
var evolveTsToDate = R.evolve({
  ts: tsToDate
});
exports.evolveTsToDate = evolveTsToDate;