"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.prettyPrint = exports.highlightedText = exports.brighterThan = exports.brightness = exports.prettifyNumber = exports.prettifyNumberWithOptions = exports.prettifyInfoValue = exports.camelToLabel = exports.unCamel = exports.toTitle = exports.typeToUnit = exports.noneOptionKey = exports.contrastThreshold = void 0;

var R = _interopRequireWildcard(require("ramda"));

var _lodash = _interopRequireDefault(require("lodash.round"));

var _lodash2 = _interopRequireDefault(require("lodash.floor"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

/**
 * Utils for visual components. Anything with a purpose to aid in showing something to a user.
 * @module ui-utils
 * @typicalname uiUtils
 */
var contrastThreshold = 145;
exports.contrastThreshold = contrastThreshold;
var noneOptionKey = '_%NONE%_';
exports.noneOptionKey = noneOptionKey;
var typeToUnit = {
  temperature: '°C',
  weight: 'kg',
  force: 'N',
  length: 'mm',
  pressure: 'bar'
};
exports.typeToUnit = typeToUnit;

var toTitle = function toTitle(str) {
  return str.replace(/^./g, function (_char) {
    return _char.toUpperCase();
  });
};

exports.toTitle = toTitle;

var unCamel = function unCamel(str) {
  return str.replace(/([A-Z])/g, function (all, ch) {
    return " ".concat(ch.toLowerCase());
  });
};

exports.unCamel = unCamel;
var camelToLabel = R.compose(toTitle, unCamel); // export default toTitle;

exports.camelToLabel = camelToLabel;

var hexToRgb = function hexToRgb(hex) {
  var r;
  var g;
  var b;

  if (hex) {
    var color = hex.charAt(0) === '#' ? hex.substr(1) : hex;
    r = color.charAt(0) + color.charAt(1);
    g = color.charAt(2) + color.charAt(3);
    b = color.charAt(4) + color.charAt(5);
    r = parseInt(r, 16);
    g = parseInt(g, 16);
    b = parseInt(b, 16);
  }

  return {
    r: r,
    g: g,
    b: b
  };
};

var prettifyInfoValue = R.when(R.is(Object), R.ifElse(R.has('type'), function (v) {
  return "".concat(v.pl, " ").concat(typeToUnit[v.type]);
}, R.prop('pl')));
exports.prettifyInfoValue = prettifyInfoValue;

var prettifyNumberWithOptions = function prettifyNumberWithOptions(_ref) {
  var precision = _ref.precision,
      decimals = _ref.decimals,
      ltOnePrecision = _ref.ltOnePrecision,
      _ref$toFloor = _ref.toFloor,
      toFloor = _ref$toFloor === void 0 ? false : _ref$toFloor;
  return R.ifElse(R.o(Number.isNaN, Number), R.identity, R.compose(function (x) {
    return x.toLocaleString();
  }, function (x) {
    // Decimals
    if (R.isNil(decimals)) return x;
    var d = R.is(Function, decimals) ? decimals(x) : decimals;
    return toFloor ? (0, _lodash2["default"])(x, d) : (0, _lodash["default"])(x, d);
  }, function (x) {
    // Significant digits / precision
    if (R.isNil(precision)) return x;
    var prec = x < 1 && !R.isNil(ltOnePrecision) ? ltOnePrecision : precision;
    var p = R.is(Function, prec) ? prec(x) : prec;
    return Number(x.toPrecision(p));
  }, Number));
}; // return (x < 1
//   ? x.toPrecision(ltOnePrecision)
//   : Number(x.toPrecision(R.is(Function, precision) ? precision(x) : precision)).toFixed(
//       R.is(Function, decimals) ? decimals(x) : decimals,
//     )).toLocaleString();


exports.prettifyNumberWithOptions = prettifyNumberWithOptions;
var prettifyNumber = prettifyNumberWithOptions({
  ltOnePrecision: 1,
  precision: 3,
  decimals: function decimals(x) {
    return x >= 100 ? 0 : 1;
  }
}); // R.when(R.is(Number), x =>
//   x < 1 ? x.toPrecision(1) : Number(x.toPrecision(3)).toFixed(x >= 100 ? 0 : 1),
// );

exports.prettifyNumber = prettifyNumber;

var brightness = function brightness(hex) {
  var _hexToRgb = hexToRgb(hex),
      r = _hexToRgb.r,
      g = _hexToRgb.g,
      b = _hexToRgb.b;

  return (r * 299 + g * 587 + b * 114) / 1000;
};

exports.brightness = brightness;

var brighterThan = function brighterThan(hex, x) {
  return brightness(hex) > x;
};

exports.brighterThan = brighterThan;

var highlightedText = function highlightedText(primary, secondary) {
  var bg = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '#FFFFFF';
  var threshold = brightness(bg) - (255 - contrastThreshold);
  var textColor = primary;
  if (brighterThan(textColor, threshold)) textColor = secondary;
  if (brighterThan(textColor, threshold)) textColor = 'black';
  return textColor;
}; // Prettier text of entity props


exports.highlightedText = highlightedText;

var prettyPrint = function prettyPrint(item) {
  if (R.is(Object, item)) {
    return JSON.stringify(item);
  } // if (Array.isArray(item)) {
  //   const val = item.map(i => prettyPrint(i));
  //   return `${val.join(', ')}`;
  // }
  // if (R.is(Object, item)) {
  //   const row = R.toPairs(item).reduce((arr, v, k) => arr.concat(`${k}: ${v}`), []);
  //   return `{ ${row.join(', ')} }`;
  // }


  if (typeof item === 'number') return item;
  return item ? item.toString() : false;
};

exports.prettyPrint = prettyPrint;