"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Chart = void 0;

var _react = _interopRequireWildcard(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _chart = _interopRequireDefault(require("chart.js"));

var R = _interopRequireWildcard(require("ramda"));

var _transforms = require("../transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj["default"] = obj; return newObj; } }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

// import { generateString } from '../data-utils';
var propTypes = {
  data: _propTypes["default"].objectOf(_propTypes["default"].any),
  style: _propTypes["default"].objectOf(_propTypes["default"].any),
  type: _propTypes["default"].string,
  options: _propTypes["default"].objectOf(_propTypes["default"].any),
  values: _propTypes["default"].arrayOf(_propTypes["default"].any),
  point: _propTypes["default"].objectOf(_propTypes["default"].any),
  length: _propTypes["default"].number
};
var defaultProps = {
  style: {},
  data: undefined,
  values: undefined,
  type: 'line',
  options: {},
  point: {},
  length: 10
};
var pointRadius = 1.3;
var chartColors = ['rgb(50, 152, 235)', 'rgb(255, 159, 64)', 'rgb(75, 185, 92)', 'rgb(153, 102, 255)', 'rgb(255, 99, 99)', 'rgb(255, 205, 86)', 'rgb(190, 90, 60)', 'rgb(231, 80, 200)', 'rgb(60, 203, 180)', 'rgb(255, 20, 0)', 'rgb(201, 203, 207)'];
var defaultCanvasStyle = {
  width: 300,
  height: 300
};
var commonOptions = {
  animation: {
    duration: 200
  },
  legend: {
    labels: {
      boxWidth: 6
    }
  }
};

var getDefaultDataset = function getDefaultDataset(idx) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
  return {
    fill: false,
    label: "pl[".concat(idx, "]"),
    data: data,
    backgroundColor: chartColors[idx],
    borderColor: chartColors[idx],
    pointRadius: pointRadius
  };
};

var defaultOptions = {
  bar: _objectSpread({}, commonOptions),
  line: _objectSpread({}, commonOptions, {
    // tooltips: { enabled: false },
    scales: {
      xAxes: [{
        type: 'time',
        time: {
          parser: 'HH:mm:ss',
          tooltipFormat: 'd MMM HH:mm:ss',
          minUnit: 'second',
          displayFormats: {
            millisecond: 'mm:ss.SSS',
            second: 'mm:ss',
            minute: 'H:mm',
            hour: 'M-d H'
          }
        },
        scaleLabel: {
          display: false // labelString: 'Date',

        }
      }],
      yAxes: [{
        scaleLabel: {
          display: false // labelString: 'value',

        }
      }]
    }
  })
};

var adjustDatasetsToPayload = function adjustDatasetsToPayload(datasets, payload) {
  while (datasets.length < payload.length) {
    datasets.push(getDefaultDataset(datasets.length, R.repeat(null, datasets[0] ? datasets[0].data.length : 0)));
  }

  while (datasets.length > payload.length) {
    datasets.pop();
  }
};

var pointIntoChart = function pointIntoChart(chart, length, point) {
  var x = point.x || point.ts || point.timestamp || point.time || new Date();
  var ys = point.payload || point.pl || point.data || [null]; // Add or remove datasets depending on number of ys:

  adjustDatasetsToPayload(chart.data.datasets, ys); // chart.update();
  // Calculate if we should cut the tail

  var remove = true;
  var chronologic = (0, _transforms.nilOrEmpty)(chart.data.labels) || x > chart.data.labels[0];
  if (chronologic && chart.data.labels.length < length) remove = false; // Add the new data point (to labels & datasets)

  if (remove) chart.data.labels.shift();
  chart.data.labels.push(x);
  chart.data.datasets.forEach(function (ds, ix) {
    var val = ys[ix]; // if (!R.is(Object, val) && !R.is(Number, val) && !R.isNil(val)) val = { data: val };
    // if (R.is(Object, val)) val.valueOf = () => 0;

    ds.data.push(val);
    if (remove) ds.data.shift();
  });
};

var fixDatasetStyle = function fixDatasetStyle(ds, ix) {
  /* eslint-disable no-param-reassign */
  R.mapObjIndexed(function (val, key) {
    ds[key] = ds[key] || val;
  })(getDefaultDataset(ix, ds.data)); // if (!ds.backgroundColor) ds.backgroundColor = chartColors[ix];
  // if (!ds.borderColor) ds.borderColor = chartColors[ix];
  // if (!ds.pointRadius) ds.pointRadius = pointRadius;

  /* eslint-enable no-param-reassign */
};

var Chart = function Chart(_ref) {
  var style = _ref.style,
      type = _ref.type,
      data = _ref.data,
      options = _ref.options,
      point = _ref.point,
      length = _ref.length,
      values = _ref.values,
      rest = _objectWithoutProperties(_ref, ["style", "type", "data", "options", "point", "length", "values"]);

  var canvasRef = (0, _react.useRef)(null);
  var chart = (0, _react.useRef)(null); // const chartId = useRef(generateString(8));

  (0, _react.useEffect)(function () {
    // console.log('Chart init:', type, data);
    if (!chart.current) {
      /* eslint-disable no-param-reassign */
      data = data || {};
      if (!Array.isArray(data.datasets)) data.datasets = [];
      /* eslint-enable no-param-reassign */

      data.datasets.forEach(fixDatasetStyle);
      chart.current = new _chart["default"](canvasRef.current, {
        type: type,
        data: data,
        options: R.mergeDeepLeft(options, defaultOptions[type])
      });
    }
  }, []);
  (0, _react.useEffect)(function () {
    // console.log('Chart update:', length, point);
    if ((0, _transforms.nilOrEmpty)(point)) return; // if (Array.isArray(point)) {
    //   R.map(payloadIntoChart(chart.current, length))(payload);

    pointIntoChart(chart.current, length, point); // console.log('Chart datasets:', chart.current.data.datasets);
    // console.log('Chart labels:', chart.current.data.labels);

    chart.current.update();
  }, [point]);
  (0, _react.useEffect)(function () {
    if ((0, _transforms.nilOrEmpty)(values)) return;
    chart.current.data.datasets[0].data = values;
  }, [values]);

  var canvasStyle = _objectSpread({}, defaultCanvasStyle, {}, style); // canvasStyle.minHeight = canvasStyle.height;


  return (0, _react.useMemo)(function () {
    return _react["default"].createElement("canvas", _extends({
      // id={chartId}
      ref: canvasRef,
      width: canvasStyle.width,
      height: canvasStyle.height,
      style: canvasStyle
    }, rest));
  }, [chart.current]);
};

exports.Chart = Chart;
Chart.propTypes = propTypes;
Chart.defaultProps = defaultProps;