"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Units = exports.defaultUnits = void 0;

var _jsQuantities = _interopRequireDefault(require("js-quantities"));

var _transforms = require("./transforms");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { "default": obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var defaultUnits = {
  // kind: ['metric', 'imperial', 'option #3', 'option #4', ...]
  length: [{
    name: 'Meters',
    abbv: 'm'
  }, {
    name: 'Feets',
    abbv: 'ft'
  }],
  area: [{
    name: 'Square meters',
    abbv: 'm2'
  }, {
    name: 'Square feets',
    abbv: 'ft2'
  }],
  volume: [{
    name: 'Liters',
    abbv: 'l'
  }, {
    name: 'Fluid-ounces',
    abbv: 'floz'
  }],
  temperature: [{
    name: 'Celsius',
    abbv: 'tempC'
  }, {
    name: 'Fahrenheit',
    abbv: 'tempF'
  }],
  pressure: [{
    name: 'Pascal',
    abbv: 'MPa'
  }, {
    name: 'psi',
    abbv: 'psi'
  }, {
    name: 'Bars',
    abbv: 'bar'
  }]
}; // const kindToAbbv = (units, unitsConf, kind) =>
// LocalStorage helpers.

exports.defaultUnits = defaultUnits;

var defaultSave = function defaultSave(obj) {
  return localStorage.setItem('units', JSON.stringify(obj));
};

var defaultLoad = function defaultLoad() {
  try {
    return JSON.parse(localStorage.getItem('units')) || {};
  } catch (_unused) {
    return {};
  }
};
/**
 * Implements a wrapper for the js-quantities library.
 *
 * @param {Object} units Object specifying kinds and units.
 * @param {function} save Callback to save unit data.
 * @param {function} load Callback to load unit data.
 */


var Units = function Units(units, save, load) {
  var _units = units || defaultUnits;

  var _save = save || defaultSave;

  var _load = load || defaultLoad;

  var getKind = function getKind(kind) {
    var value = _load();

    return value.kinds ? value.kinds[kind] : undefined;
  };

  var representKind = function representKind(metric, kind) {
    var toUnit = getKind(kind);
    if (toUnit) return toUnit;
    if (_units[kind]) return _units[kind][metric ? 0 : 1].abbv;
    return undefined;
  };

  var isMetric = function isMetric() {
    return _load().isMetric === true;
  };

  var represent = function represent(metric) {
    return function (val, unit) {
      var q = (0, _jsQuantities["default"])(1, unit);
      var toUnit = representKind(metric === undefined ? isMetric() : metric, q.kind()) || q.toBase().units();

      var convert = _jsQuantities["default"].swiftConverter(unit, toUnit);

      return {
        value: convert(val),
        unit: toUnit
      }; // return toUnit ? q.to(toUnit) : q.toBase().units();
    };
  };

  var as = represent();

  var getUnit = function getUnit(fromUnit) {
    return as(1, fromUnit).units();
  };

  return {
    setMetric: function setMetric(metric) {
      return _save(_objectSpread({}, _load(), {
        isMetric: metric
      }));
    },
    isMetric: isMetric,
    // Kind option accessors
    setKind: function setKind(kind, option) {
      var value = _load();

      if ((0, _transforms.nilOrEmpty)(option)) {
        if (value.kinds) delete value.kinds[kind];
      } else {
        if (!value.kinds) value.kinds = {};
        value.kinds[kind] = option;
      }

      _save(value);
    },
    getKind: getKind,
    // (value, 'unit') => value.
    asMetric: represent(true),
    asImperial: represent(false),
    as: as,
    // Get user specific unit from unit or kind.
    getUnit: getUnit,
    getUnitByKind: function getUnitByKind(kind) {
      return getUnit(_jsQuantities["default"].getUnits(kind)[0]);
    },
    fromBase: function fromBase(value, kind) {
      return as(value, (0, _jsQuantities["default"])(1, _jsQuantities["default"].getUnits(kind)[0]).toBase().units());
    },
    getUnits: function getUnits() {
      return _units;
    }
  };
};

exports.Units = Units;